#Written By: Adam Roberts
#Converts standard SAM and FASTA input into the mseq input format

from numpy import *

path_to_fasta = 'hg18-transcripts.fa'
path_to_sam = 'bowtie_out.sam'

# Read in sequences
print 'read in sequences'
f = file(path_to_fasta,'r')
line = f.readline()
seqs = dict()
while line != '':
	if line[0] == '>':
		name  = line.split()[0][1:]
		seqs[name] = ''
	else:
		seqs[name] += line.replace('\n','')
	line = f.readline()

# Get counts
print 'calculating raw counts'
f = file(path_to_sam, 'r')
counts = dict()
line = f.readline()
while line[0] == '@':
	line = f.readline()

while line != '':
	col = line.split('\t')
	name = col[2]
	counts[name] = counts.get(name, 0) + 1
	line = f.readline()


# Calculate RPKM
print 'calculate rpkm'
rpkm = dict()
for name, count in counts.iteritems():
	if name[0:13] == 'hg18_refGene_':
		rpkm[name] = float(count)/len(seqs[name])
	line = f.readline()

# Find Top 100
print 'find top 100'
name_list = []
rpkm_list = []
i = 0
for name, r in rpkm.iteritems():
	rpkm_list.append(r)
	name_list.append(name)

name_array = array(name_list)
rpkm_array = array(rpkm_list)
order = argsort(rpkm_array)

top100 = name_array.take(order[-100:])[::-1]

#Find TAQ Genes
print 'find taq genes'
f = file('/Users/adarob/experiments/maqc-brain/taq.csv','r')
f.readline()
line = f.readline()
taq = []
while line != '':
	taq.append('hg18_refGene_'+line.split(',')[0])
	line = f.readline()

# Create Histograms for Top 100 AND Taq Genes
print 'create histograms'
top100_counts = dict()
taq_counts = dict()
for name in top100:
	top100_counts[name] = zeros((2,len(seqs[name])))
for name in taq:
	taq_counts[name] = zeros((2,len(seqs[name])))

f = file('/Volumes/scratch/adams_maps/illumina-brain/bowtie_out.sam','r')
line = f.readline()
while line[0] == '@':
	line = f.readline()

while line != '':
	col = line.split('\t')
	name = col[2]
	if name in top100 or name in taq:
		strand = (int(col[1]) & 0x0010) != 0
		start = int(col[3])-1
		if strand: #reverse
			start = start + 49
		
		if name in top100:
			top100_counts[name][strand,start] += 1
		if name in taq:
			taq_counts[name][strand,start] += 1
			
	line = f.readline()

# Output Taq and Top 100 in mseq format
print 'output in mseq format'
f = file('top100_counts.csv','w')
f.write('index,tag,seq,count\n')
for index in xrange(len(top100)):
	name = top100[index]
	seq = seqs[name]
	counts = top100_counts[name]

	for i in xrange(len(seq)):
		f.write('%d,%d,%s,%d\n' % (index, (seq[i].islower() or i < 100 or i >= len(seq)-100), seq[i].upper(), counts[0,i]))
	for i in xrange(len(seq)-1,-1,-1):
		f.write('%d,%d,%s,%d\n' % (index, (seq[i].islower() or i < 100 or i >= len(seq)-100), seq[i].upper(), counts[1,i]))
f.close()

f = file('taq_counts.csv','w')
f.write('index,tag,seq,count\n')
for index in xrange(len(taq)):
	name = taq[index]
	seq = seqs[name]
	counts = taq_counts[name]
	
	for i in xrange(len(seq)):
		f.write('%d,%d,%s,%d\n' % (index, (seq[i].islower() or i < 100 or i >= len(seq)-100), seq[i].upper(), counts[0,i]))
	for i in xrange(len(seq)-1,-1,-1):
		f.write('%d,%d,%s,%d\n' % (index, (seq[i].islower() or i < 100 or i >= len(seq)-100), seq[i].upper(), counts[1,i]))
f.close()
